from __future__ import division

import random
import sys

try: import numpy
except ImportError:
    print "Sorry, numpy required."
    sys.exit()

from controller import *
from view import *

def intersect(shapenode1,shapenode2,filter):
    min1 = numpy.dot(shapenode1.shape.min,shapenode1.absolute_transform)[:3]*filter
    max1 = numpy.dot(shapenode1.shape.max,shapenode1.absolute_transform)[:3]*filter
    min2 = numpy.dot(shapenode2.shape.min,shapenode2.absolute_transform)[:3]*filter
    max2 = numpy.dot(shapenode2.shape.max,shapenode2.absolute_transform)[:3]*filter
    return numpy.all(min1<=max2) and numpy.all(max1>=min2)

def collide(shapenode1,shapenode2,filter):
    normal = numpy.array([0.0,0.0,0.0])
    delta = (shapenode2.absolute_transform[3,:3]-shapenode1.absolute_transform[3,:3])*filter
    overlap = ((shapenode1.shape.size+shapenode2.shape.size)*filter)/2 - numpy.absolute(delta)
    collision = numpy.all(overlap>=numpy.array([0.0,0.0,0.0]))
    if collision:
        dim = numpy.where(overlap==numpy.nanmin(overlap/filter))[0][0]
        normal[dim] = -delta[dim]
        normal /= numpy.linalg.norm(normal)
    return collision,normal,delta

class SplashView(GroupView):
    def __init__(self):
        GroupView.__init__(self,pygame.Rect(0,0,640,480))
        self.time = 4000.0
        self.initialize()
    def initialize(self):
        test = TestView(pygame.Rect(0,0,640,480))
        test.scene.color = (0,0,0)
        #test.fill = False
        #test.stroke = True
        #test.light = False
        #test.cull = 2
        self.append(test)
        url = TextView("www.the-source.de",(255,255,255))
        url.viewport.centerx = self.viewport.centerx
        url.viewport.centery = self.viewport.centery + 100
        self.append(url)
    def update(self,delta):
        GroupView.update(self,delta)
        self.time -= delta
        if self.time<=0.0:
            controller = self.controller
            controller.pop()
            controller.push(GameView())
    def handle(self,event): pass

class GameView(GroupView):
    def __init__(self):
        GroupView.__init__(self,pygame.Rect(0,0,640,480))
        self.sound = pygame.mixer.Sound('click.wav')
        self.channel = pygame.mixer.Channel(0)
        self.paused = True
        self.stopped = False
        self.orientations = [
            quaternion.rotation(0.0,numpy.array([0.0,1.0,0.0])),
            quaternion.rotation(90.0,numpy.array([0.0,1.0,0.0])),
            quaternion.rotation(180.0,numpy.array([0.0,1.0,0.0])),
            quaternion.rotation(270.0,numpy.array([0.0,1.0,0.0])),
        ]
        self.orientation = 0
        self.filter = numpy.roll(numpy.array([True,True,False]),(self.orientation%2))
        self.interpolator = Interpolator(1000)
        self.lives = 3
        self.points = 0
        self.state = 0
        def level00(bricks):
            brick = ShapeNode(CuboidShape(80.0,40.0,80.0),numpy.array([255,255,255]))
            bricks.append(brick)
        def level01(bricks):
            for i in [-2,-1,0,1,2]:
                for j in [-1,1]:
                    for k in [-1,1]:
                        brick = ShapeNode(CuboidShape(80.0,40.0,80.0),(numpy.array([i,j,k])+2)*255/5)
                        brick.translation = numpy.array([i*80.0,j*40.0,k*80.0])
                        bricks.append(brick)
        def level02(bricks):
            for i in [-2,-1,0,1,2]:
                for j in [-1,0,1]:
                    for k in [-2,-1,0,1,2]:
                        if (i==0 and j==0) or (j==0 and k==0) or (k==0 and i==0):
                            brick = ShapeNode(CuboidShape(80.0,40.0,80.0),(numpy.array([i,j,k])+2)*255/5)
                            brick.translation = numpy.array([i*80.0,j*40.0,k*80.0])
                            bricks.append(brick)
        def level03(bricks):
            for i in [-2,-1,0,1,2]:
                for j in [-1,0,1]:
                    for k in [-2,-1,0,1,2]:
                        if not ((abs(i)<2 or abs(k)<2) and abs(j)==0) and not ((abs(i)<2 and abs(k)<2) and abs(j)==1):
                            if not (abs(i+1)%2 and abs(j)==1 and abs(k+1)%2):
                                brick = ShapeNode(CuboidShape(80.0,40.0,80.0),(numpy.array([i,j,k])+2)*255/5)
                                brick.translation = numpy.array([i*80.0,j*40.0,k*80.0])
                                bricks.append(brick)
        def level04(bricks):
            for i in [-2,-1,0,1,2]:
                for j in [-1,0,1]:
                    for k in [-2,2]:
                        if (i+j)%2 and k>0 or not ((i+j)%2 or k>0):
                            brick = ShapeNode(CuboidShape(80.0,40.0,80.0),(numpy.array([i,j,k])+2)*255/5)
                            brick.translation = numpy.array([i*80.0,j*40.0,k*80.0])
                            bricks.append(brick)
        self.levels = [
            #level00,
            level01,
            level02,
            level03,
            level04
        ]
        self.level = 0
        self.initialize()
        self.advance()
    def initialize(self):
        self.text_lives = TextView("Lives: " + str(self.lives))
        self.text_lives.viewport.top = self.viewport.top + 5
        self.text_lives.viewport.centerx = self.viewport.centerx - 160
        self.append(self.text_lives)
        self.text_points = TextView("Points: " + str(self.points))
        self.text_points.viewport.top = self.viewport.top + 5
        self.text_points.viewport.centerx = self.viewport.centerx
        self.append(self.text_points)
        self.text_levels = TextView("Level: " + str(self.level))
        self.text_levels.viewport.top = self.viewport.top + 5
        self.text_levels.viewport.centerx = self.viewport.centerx + 160
        self.append(self.text_levels)
        self.game = SceneView(pygame.Rect(40,30,560,420))
        self.game.scene.color = numpy.array([31,31,63])
        self.game.scene.light = numpy.array([0.0,0.0,-1.0,0.0])
        self.game.scene.light /= numpy.linalg.norm(self.game.scene.light)
        self.game.scene.camera = CameraNode()
        whalf,hhalf = self.game.viewport.size[0]/2,self.game.viewport.size[1]/2
        self.game.scene.camera.project = matrix.ortho(-whalf,whalf,-hhalf,hhalf,0.0,1000.0)
        self.game.scene.camera.translation = numpy.array([0.0,0.0,500.0])
        self.orbit = GroupNode()
        self.orbit.orientation = self.orientations[self.orientation]
        self.orbit.append(self.game.scene.camera)
        self.game.scene.root.append(self.orbit)
        self.ball = ShapeNode(CuboidShape(10.0,10.0,10.0),numpy.array([255,255,255]))
        self.game.scene.root.append(self.ball)
        self.bricks = GroupNode()
        self.bricks.translation = numpy.array([0.0,120.0,0.0])
        self.game.scene.root.append(self.bricks)
        self.paddle = ShapeNode(CuboidShape(80.0,20.0,80.0),numpy.array([255,255,0]))
        self.paddle.translation = numpy.array([0.0,-160.0,0.0])
        self.game.scene.root.append(self.paddle)
        self.floor = ShapeNode(CuboidShape(560.0,20.0,560.0),numpy.array([255,0,0]))
        self.floor.translation = numpy.array([0.0,-210.0,0.0])
        self.game.scene.root.append(self.floor)
        def initialize_walls(size,width):
            offset = (size[0]+width)/2,(size[1]+width)/2
            self.lt = ShapeNode(CuboidShape(width,size[1],size[0]),numpy.array([0,255,0]))
            self.rt = ShapeNode(CuboidShape(width,size[1],size[0]),numpy.array([0,255,0]))
            self.tp = ShapeNode(CuboidShape(size[0],width,size[0]),numpy.array([0,255,0]))
            self.bt = ShapeNode(CuboidShape(size[0],width,size[0]),numpy.array([0,255,0]))
            self.ft = ShapeNode(CuboidShape(size[0],size[1],width),numpy.array([0,255,0]))
            self.bk = ShapeNode(CuboidShape(size[0],size[1],width),numpy.array([0,255,0]))
            self.lt.translation = numpy.array([offset[0],0.0,0.0])
            self.rt.translation = numpy.array([-offset[0],0.0,0.0])
            self.tp.translation = numpy.array([0.0,offset[1],0.0])
            self.bt.translation = numpy.array([0.0,-offset[1],0.0])
            self.ft.translation = numpy.array([0.0,0.0,offset[0]])
            self.bk.translation = numpy.array([0.0,0.0,-offset[0]])
        initialize_walls(self.game.viewport.size,50)
        self.append(self.game)
    def appear(self):
        pygame.mouse.set_visible(False)
        pygame.event.set_grab(True)
        self.paused = False
        GroupView.appear(self)
    def disappear(self):
        GroupView.disappear(self)
        self.paused = True
        pygame.event.set_grab(False)
        pygame.mouse.set_visible(True)
    def transition_start(self):
        pygame.event.set_blocked(MOUSEMOTION)
        self.stopped = True
    def transition_end(self):
        self.stopped = False
        pygame.event.set_allowed(MOUSEMOTION)
    def advance(self):
        self.levels[self.level](self.bricks)
        self.text_levels.set("Level: " + str(self.level))
        self.level += 1
    def handle(self,event):
        GroupView.handle(self,event)
        if event.type==KEYDOWN:
            if event.key == K_ESCAPE: self.controller.push(PauseView())
        if event.type==MOUSEBUTTONUP:
            if event.button==3:
                def adjust_orbit(q0,q1):
                    def f(t): self.orbit.orientation = quaternion.slerp(q0,q1,t)
                    return f
                next = (self.orientation+1)%len(self.orientations)
                self.interpolator.action = adjust_orbit(self.orbit.orientation,self.orientations[next])
                self.interpolator.started = lambda:self.transition_start()
                self.interpolator.stopped = lambda:self.transition_end()
                self.interpolator.stop()
                self.interpolator.start()
                self.orientation = next
                self.filter = numpy.roll(numpy.array([True,True,False]),(self.orientation%2))
        if event.type==MOUSEMOTION:
            self.paddle.translation += numpy.roll(numpy.array([event.rel[0],0.0,0.0]),-(self.orientation%2))*(-1,1)[self.orientation<2]
            whalf = (self.game.viewport.size[0]-self.paddle.shape.size[0])/2
            self.paddle.translation = self.paddle.translation.clip(-whalf,whalf)
        if self.state==0:
            if event.type==MOUSEBUTTONUP:
                if event.button==1:
                    self.ball.translational_velocity = numpy.array([0.0,100.0,0.0]) + numpy.array([random.randint(-45,45),0.0,random.randint(-45,45)])
                    self.state = 1
    def update(self,delta):
        if self.paused: return
        self.interpolator.update(delta)
        if self.stopped: return
        if self.state==0:
            self.ball.translation = self.paddle.translation + numpy.array([0.0,15.0,0.0])
        if self.state==1:
            if intersect(self.ball,self.floor,self.filter):
                self.state = 0
                self.ball.translational_velocity = numpy.array([0.0,0.0,0.0])
                self.lives -= 1
                self.text_lives.set("Lives: " + str(self.lives))
            collision,n,d = collide(self.ball,self.paddle,self.filter)
            if collision:
                if numpy.dot(self.ball.translational_velocity,n)<0:
                    offset = d*numpy.array([True,False,True])
                    self.ball.translational_velocity = numpy.dot(self.ball.translational_velocity,matrix.reflection(n)[:3,:3]) - offset
                    if numpy.linalg.norm(self.ball.translational_velocity)<200: self.ball.translational_velocity *= 1.1
                    self.channel.play(self.sound)
            for wall in [self.lt,self.rt,self.tp,self.ft,self.bk]:
                collision,n,d = collide(self.ball,wall,numpy.array([True,True,True]))
                if collision: 
                    if numpy.dot(self.ball.translational_velocity,n)<0:
                        self.ball.translational_velocity = numpy.dot(self.ball.translational_velocity,matrix.reflection(n)[:3,:3])
            candidates = []
            for brick in self.bricks:
                collision,n,d = collide(self.ball,brick,self.filter)
                if collision: candidates.append((brick,n,d))
            if len(candidates):
                candidates.sort(key=lambda (brick,n,d):brick.absolute_transform[3,(2,0)[self.orientation%2]],reverse=self.orientation==0 or self.orientation==3)
                brick,n,d = candidates[0]
                if numpy.dot(self.ball.translational_velocity,n)<0:
                    self.ball.translational_velocity = numpy.dot(self.ball.translational_velocity,matrix.reflection(n)[:3,:3])
                    self.bricks.remove(brick)
                    self.points += 50
                    self.text_points.set("Points: " + str(self.points))
                    self.channel.play(self.sound)
            if not len(self.bricks) and self.level<len(self.levels):
                self.advance()
                self.state = 0
                self.ball.translational_velocity = numpy.array([0.0,0.0,0.0])
            if not (self.lives and len(self.bricks)):
                self.controller.push(GameOverView())
        GroupView.update(self,delta)

class PauseView(View):
    def __init__(self):
        View.__init__(self,pygame.Rect(0,0,640,480))
        self.initialize()
    def initialize(self):
        pygame.font.init()
        text = pygame.font.Font(None,24).render("Pause",1,(255,255,255))
        rect = text.get_rect()
        rect.center = self.surface.get_rect().center
        self.surface.fill((0,0,0,0))
        self.surface.blit(text,rect.topleft)
    def handle(self,event):
        View.handle(self,event)
        if event.type==KEYDOWN:
            if event.key == K_ESCAPE: self.controller.pop()

class GameOverView(View):
    def __init__(self):
        View.__init__(self,pygame.Rect(0,0,640,480))
        self.initialize()
    def initialize(self):
        pygame.font.init()
        text = pygame.font.Font(None,24).render("Game Over",1,(255,255,255))
        rect = text.get_rect()
        rect.center = self.surface.get_rect().center
        self.surface.fill((0,0,0,0))
        self.surface.blit(text,rect.topleft)

def main():
    controller = Controller("Blatt10",(640,480))
    controller.push(SplashView())
    controller.run()

if __name__ == '__main__':
    main()
'''
import hotshot, hotshot.stats

if __name__ == '__main__':
    prof = hotshot.Profile("hotshot_blatt10_stats")
    prof.runcall(main)
    prof.close()
    hotshot.stats.load("hotshot_blatt10_stats").sort_stats("time").print_stats()
'''