from serpent.document import *

"""
OBJ Format
"""

class OBJGrammar(Grammar):
    def __init__(self):
        Grammar.__init__(self)
        newline = RR('\s*') | TR('\n')
        space = RR('[^\S\n]*') | TR(' ')
        int_number = RR('\d+')
        float_number = RR('[+-]?\d+(\.\d+)?')
        comment = (TR('#') << RR('.*')) | RR('\s+')
        vertex = (TR('v') << space << float_number['x':float:str] << space << float_number['y':float:str] << space << float_number['z':float:str])^['vertex']
        group = (TR('g') << RR('.*'))^['group']
        face = (TR('f') << space << int_number['x':int:str] << space << int_number['y':int:str] << space << int_number['z':int:str])^['face']
        vertices = (vertex << newline)*['vertices']
        faces = (face << newline)*['faces']
        file = ((comment | vertex | group | face) << newline)*['items']
        self.rule = file

class OBJTransform(Transform):
    def toInternal(self,data):
        return {
            'vertices': [[vertex['x'],vertex['y'],vertex['z']] for vertex in Sequence(data,'vertex','items')],
            'faces': [[face['x'],face['y'],face['z']] for face in Sequence(data,'face','items')]
        }
    def fromInternal(self,data):
        raise TransformException()

class OBJDocument(Document):
    Document.formats['.obj'] = lambda:OBJDocument()
    def __init__(self):
        Document.__init__(self,OBJGrammar(),OBJTransform())