import math
import numpy

'''
Mesa CVS:
http://webcvs.freedesktop.org/mesa/Mesa/
Quaternion to matrix:
http://www.lfd.uci.edu/~gohlke/code/transformations.py.html
'''

def identity():
    return numpy.array([
        [1.0,    0.0,    0.0,    0.0],
        [0.0,    1.0,    0.0,    0.0],
        [0.0,    0.0,    1.0,    0.0],
        [0.0,    0.0,    0.0,    1.0]
    ])

def scale(v):
    return numpy.array([
        [v[0],   0.0,    0.0,    0.0],
        [0.0,   v[1],    0.0,    0.0],
        [0.0,    0.0,   v[2],    0.0],
        [0.0,    0.0,    0.0,    1.0]
    ])

def orientation(quaternion):
    q = quaternion.copy()
    q *= math.sqrt(2.0/numpy.dot(q,q))
    q = numpy.outer(q,q)
    return numpy.array([
        [1.0-q[1,1]-q[2,2],     q[0,1]-q[2,3],     q[0,2]+q[1,3], 0.0],
        [    q[0,1]+q[2,3], 1.0-q[0,0]-q[2,2],     q[1,2]-q[0,3], 0.0],
        [    q[0,2]-q[1,3],     q[1,2]+q[0,3], 1.0-q[0,0]-q[1,1], 0.0],
        [              0.0,               0.0,               0.0, 1.0]
    ])

def translation(v):
    return numpy.array([
        [1.0,    0.0,    0.0,    0.0],
        [0.0,    1.0,    0.0,    0.0],
        [0.0,    0.0,    1.0,    0.0],
        [v[0],  v[1],   v[2],    1.0]
    ])

def reflection(v):
    return numpy.array([
        [-v[0]*v[0]+v[1]*v[1]+v[2]*v[2], -2*v[1]*v[0],                  -2*v[2]*v[0],                  0.0],
        [-2*v[0]*v[1],                   v[0]*v[0]-v[1]*v[1]+v[2]*v[2], -2*v[2]*v[1],                  0.0],
        [-2*v[0]*v[2],                   -2*v[1]*v[2],                  v[0]*v[0]+v[1]*v[1]-v[2]*v[2], 0.0],
        [0.0,                            0.0,                           0.0,                           1.0]
    ])

def ortho(l,r,b,t,n,f):
    return numpy.array([
         [2.0/(r-l),    0.0,          0.0,               0.0],
         [0.0,          2.0/(t-b),    0.0,               0.0],
         [0.0,          0.0,          -2.0/(f-n),        0.0],
         [-(r+l)/(r-l), -(t+b)/(t-b), -(f+n)/(f-n),      1.0]
    ])

def frustum(l,r,b,t,n,f):
    return numpy.array([
        [2.0*n/(r-l),   0.0,          0.0,               0.0],
        [0.0,           2.0*n/(t-b),  0.0,               0.0],
        [(r+l)/(r-l),   (t+b)/(t-b),  -(f+n)/(f-n),     -1.0],
        [0.0,           0.0,          -2.0*(f*n)/(f-n),  0.0]
    ])

def perspective(fovy,aspect,n,f):
    t = n*math.tan(fovy*math.pi/360.0)
    b = -t
    l = b*aspect
    r = t*aspect
    return frustum(l,r,b,t,n,f)