from __future__ import division

import matrix
import numpy
import quaternion

class Node:
    def __init__(self):
        self.parent = None
        self.orientation = numpy.array([0.0,0.0,0.0,1.0])
        self.translation = numpy.array([0.0,0.0,0.0])
        self.scale = numpy.array([1.0,1.0,1.0])
        self.orientational_velocity = numpy.array([0.0,0.0,0.0,1.0])
        self.translational_velocity = numpy.array([0.0,0.0,0.0])
        self.scale_velocity = numpy.array([0.0,0.0,0.0])
    def __getattr__(self,name):
        if name in ['transform']:
            if not self.__dict__.has_key('transform'):
                #value = numpy.dot(numpy.dot(matrix.scale(self.scale),matrix.translation(self.translation)),matrix.orientation(self.orientation))
                value = numpy.dot(numpy.dot(matrix.orientation(self.orientation),matrix.translation(self.translation)),matrix.scale(self.scale))
                self.__setattr__('transform',value)
        if name in ['absolute_transform']:
            if not self.__dict__.has_key('absolute_transform'):
                if self.parent is None: value = self.transform
                else: value = numpy.dot(self.transform,self.parent.absolute_transform)
                self.__setattr__('absolute_transform',value)
        try: return self.__dict__.__getitem__(name)
        except KeyError: raise AttributeError, name
    def __setattr__(self,name,value):
        if name in ['orientation','translation','scale']:
            if self.__dict__.has_key('transform'): self.__delattr__('transform')
            if self.__dict__.has_key('absolute_transform'): self.__delattr__('absolute_transform')
        if name in ['transform','parent']:
            if self.__dict__.has_key('absolute_transform'): self.__delattr__('absolute_transform')
        self.__dict__.__setitem__(name,value)
    def __delattr__(self,name):
        try: self.__dict__.__delitem__(name)
        except KeyError: raise AttributeError, name
    def update(self,delta):
        delta /= 1000
        if not numpy.all(self.orientational_velocity==numpy.array([0.0,0.0,0.0,1.0])):
            self.orientation = quaternion.mul(self.orientation,quaternion.pow(self.orientational_velocity,delta))
        if not numpy.all(self.translational_velocity==numpy.array([0.0,0.0,0.0])):
            self.translation = self.translation + self.translational_velocity*delta
        if not numpy.all(self.scale_velocity==numpy.array([0.0,0.0,0.0])):
            self.scale = self.scale + self.scale_velocity*delta
    def shape_nodes(self,nodes): pass

class GroupNode(Node):
    def __init__(self):
        Node.__init__(self)
        self.nodes = [];
    def __setattr__(self,name,value):
        if name in ['orientation','translation','scale']:
            if self.__dict__.has_key('nodes'):
                for node in self:
                    if node.__dict__.has_key('absolute_transform'): node.__delattr__('absolute_transform')
        if name in ['transform','parent']:
            if self.__dict__.has_key('nodes'):
                for node in self:
                    if node.__dict__.has_key('absolute_transform'): node.__delattr__('absolute_transform')
        Node.__setattr__(self,name,value)
    def __delattr__(self,name):
        if name in ['absolute_transform']:
            for node in self:
                if node.__dict__.has_key('absolute_transform'): node.__delattr__('absolute_transform')
        try: self.__dict__.__delitem__(name)
        except KeyError: raise AttributeError, name
    def __len__(self):
        return self.nodes.__len__()
    def __iter__(self):
        return self.nodes.__iter__()
    def append(self,node):
        node.parent = self
        self.nodes.append(node)
    def remove(self,node):
        self.nodes.remove(node)
        node.patent = None
    def update(self,delta):
        Node.update(self,delta)
        for node in self:
            node.update(delta)
    def shape_nodes(self,nodes):
        Node.shape_nodes(self,nodes)
        for node in self:
            node.shape_nodes(nodes)

class ShapeNode(Node):
    def __init__(self,shape,color):
        Node.__init__(self)
        self.shape = shape
        self.color = color
    def shape_nodes(self,nodes):
        nodes.append(self)

class CameraNode(Node):
    def __init__(self):
        Node.__init__(self)
    def projection(self):
        return numpy.dot(numpy.linalg.inv(self.absolute_transform),self.project)
    def unprojection(self):
        return numpy.dot(numpy.linalg.inv(self.project),self.absolute_transform)