import math
import numpy

def rotation(a,v):
    a *= math.pi/180.0
    ha = 0.5*a
    sa = math.sin(ha)
    ca = math.cos(ha)
    return numpy.array((sa*v[0],sa*v[1],sa*v[2],ca))

def mul(q0,q1):
    x0, y0, z0, w0 = q0
    x1, y1, z1, w1 = q1
    return numpy.array((
         x1*w0 + y1*z0 - z1*y0 + w1*x0,
        -x1*z0 + y1*w0 + z1*x0 + w1*y0,
         x1*y0 - y1*x0 + z1*w0 + w1*z0,
        -x1*x0 - y1*y0 - z1*z0 + w1*w0
    ))

def conjugate(q): return numpy.array((-q[0],-q[1],-q[2],q[3]))
def magnitude(q): return math.sqrt(numpy.dot(q,q))
def normalize(q): return mul(a,1/magnitude(q))
def inverse(q): return conjugate(q)/numpy.dot(q,q)

def exp(q):
    theta = numpy.linalg.norm(q[:3])
    sin_theta = math.sin(theta)
    cos_theta = math.cos(theta)
    result = numpy.array((q[0],q[1],q[2],cos_theta))
    if(sin_theta!=0): result[:3] *= (sin_theta/theta)
    return result

def log(q):
    theta = math.acos(q[3])
    sin_theta = math.sin(theta)
    result = numpy.array((q[0],q[1],q[2],0))
    if(sin_theta!=0): result[:3] /= (sin_theta/theta)
    return result

def pow(q,t): return exp(log(q)*t)

def slerp(q0,q1,t):
    if numpy.dot(q0,q1)<0.0: q1 *= -1.0
    return mul(q0,pow(mul(inverse(q0),q1),t))