from __future__ import with_statement

import os

from grammar import *
from transform import *

"""
Documents
"""

class DocumentException(Exception): pass

class Document:
    formats = {}
    @staticmethod
    def fromFile(file):
        try:
            key = os.path.splitext(file)[1]
            return Document.formats[key]()
        except (KeyError):
            raise DocumentException('Unable to determine document for file ("' + file + '").')
    @staticmethod
    def readFromFile(file):
        document = Document.fromFile(file)
        document.read(file)
        return document
    def __init__(self,grammar,transform):
        self.grammar = grammar
        self.transform = transform
        self.content = {}
    def __str__(self):
        if not self.content: return ''
        return self.grammar.compose(self.transform.fromInternal(self.content)) 
    def read(self,file):
        with open(file,'r') as f:content = f.read()
        self.content = self.transform.toInternal(self.grammar.parse(content))
    def write(self,file):
        content = self.grammar.compose(self.transform.fromInternal(self.content))
        with open(file,'w') as f:f.write(content)