from __future__ import division

import numpy

from serpent.document import *
from documents import *

class Shape:
    TRIANGLES = 0
    TRIANGLE_STRIP = 1
    def __init__(self,vertices,indices,mode=1):
        self.vertices = vertices
        self.indices = indices
        self.mode = mode
        self.compute_normals()
        self.compute_bounds()
    def triangles(self):
        step = (3,1)[self.mode]
        for i,n in enumerate(range(0,len(self.indices)-2,step)):
            if self.mode==1 and i%2:
                yield self.indices[n:n+3][::-1]
            else:
                yield self.indices[n:n+3]
    def compute_normals(self):
        def cross(a,b): return numpy.array([a[1]*b[2]-a[2]*b[1],a[2]*b[0]-a[0]*b[2],a[0]*b[1]-a[1]*b[0],0.0])
        self.normals = []
        for n in self.triangles():
            triangle = [self.vertices[index] for index in n]
            normal = cross(triangle[1]-triangle[0],triangle[2]-triangle[0])
            normal = normal/numpy.linalg.norm(normal)
            self.normals.append(normal)
    def compute_bounds(self):
        self.min = self.vertices.min(axis=0)
        self.max = self.vertices.max(axis=0)
        self.size = (self.max-self.min)[:3]

def RectangularShape(w=1.0,h=1.0):
    return Shape(
        numpy.array([
            [-w/2,-h/2, 0.0, 1.0],
            [ w/2,-h/2, 0.0, 1.0],
            [-w/2, h/2, 0.0, 1.0],
            [ w/2, h/2, 0.0, 1.0],
        ]),
        numpy.array([0,1,2,3])
    )

def CuboidShape(w=1.0,h=1.0,d=1.0):
    return Shape(
        numpy.array([
            [-w/2,-h/2, d/2, 1.0],
            [ w/2,-h/2, d/2, 1.0],
            [-w/2, h/2, d/2, 1.0],
            [ w/2, h/2, d/2, 1.0],
            [-w/2,-h/2,-d/2, 1.0],
            [ w/2,-h/2,-d/2, 1.0],
            [-w/2, h/2,-d/2, 1.0],
            [ w/2, h/2,-d/2, 1.0],
        ]),
        numpy.array([6,7,4,5,1,7,3,6,2,4,0,1,2,3])
    )

def OBJShape(file):
    def flatten(x):
        result = []
        for el in x:
            if isinstance(el,(list,tuple)): result.extend(flatten(el))
            else: result.append(el)
        return result
    document = Document.readFromFile(file)
    return Shape(
        numpy.array(map(lambda x:[x[0],x[1],x[2],1.0],document.content['vertices'])),
        numpy.array(map(lambda x:x-1,flatten(document.content['faces']))),
        Shape.TRIANGLES
    )